#include "btle.h"

static struct bt_xlrt_cb xlrt_cb;
static uint8_t device_info[240];
static uint8_t send_info[240];

static void on_sent(struct bt_conn *conn, void *user_data);
static ssize_t on_receive(struct bt_conn *conn,
			  const struct bt_gatt_attr *attr,
			  const void *buf,
			  uint16_t len,
			  uint16_t offset,
			  uint8_t flags);

/* XLRT UART Service Declaration */

/* UART Service Declaration 

on_receive is a local function that calls xlrt_cb.received which is knitted in 
to ble_data_received which is somehow setup from main.c to point to ble_data_received. 
It's probably done in xlrt_client_init (which is in main), but then calls
bt_nus_client_init but quite how that knits to this
instance of xlrt_cb is anyone's guess

*/
BT_GATT_SERVICE_DEFINE(xlrt_svc,
BT_GATT_PRIMARY_SERVICE(BT_UUID_XLRT_SERVICE),
	BT_GATT_CHARACTERISTIC(BT_UUID_XLRT_TX,
			       BT_GATT_CHRC_NOTIFY,
			       BT_GATT_PERM_READ,
			       NULL, NULL, device_info),
	BT_GATT_CCC(xlrt_ccc_cfg_changed, BT_GATT_PERM_READ | BT_GATT_PERM_WRITE),
	BT_GATT_CHARACTERISTIC(BT_UUID_XLRT_RX,
			       BT_GATT_CHRC_WRITE |
			       BT_GATT_CHRC_WRITE_WITHOUT_RESP,
			       BT_GATT_PERM_READ | BT_GATT_PERM_WRITE,
			       NULL, on_receive, send_info), 
);


static void xlrt_ccc_cfg_changed(const struct bt_gatt_attr *attr, uint16_t value)
{

    enum bt_xlrt_send_status status;

	if (NULL != xlrt_cb.send_enabled) 
    {
        if (BT_GATT_CCC_NOTIFY == value)        
            status = BT_XLRT_SEND_STATUS_ENABLED;
        else
            status = BT_XLRT_SEND_STATUS_DISABLED;

		LOG_DBG("Notification has been turned %s", value == BT_GATT_CCC_NOTIFY ? "on" : "off");
		xlrt_cb.send_enabled(status);
	}

}


static ssize_t on_receive(struct bt_conn *conn,
			  const struct bt_gatt_attr *attr,
			  const void *buf,
			  uint16_t len,
			  uint16_t offset,
			  uint8_t flags)
{
	LOG_DBG("Received data, handle %d, conn %p", attr->handle, (void *)conn);

	if (NULL != xlrt_cb.received)     
		xlrt_cb.received(conn, buf, len); // xlrt_cb.received is somehow tied to ble_data_received

	return len;
}

static void on_sent(struct bt_conn *conn, void *user_data)
{
	(void)user_data;
	
	if ((NULL != xlrt_cb.sent)  && (NULL != conn))
    {
		xlrt_cb.sent(conn);
	}
}


int bt_xlrt_send(struct bt_conn *conn, const uint8_t *data, uint16_t len)
{
	struct bt_gatt_notify_params params = {0};
	const struct bt_gatt_attr *attr = &xlrt_svc.attrs[2];
    int ret_val;

	params.attr = attr;
	params.data = data;
	params.len = len;
	params.func = on_sent;

	if (!conn) 
    {
		LOG_DBG("Notification send to all connected peers");
		ret_val = bt_gatt_notify_cb(NULL, &params);
	} 
    else if (bt_gatt_is_subscribed(conn, attr, BT_GATT_CCC_NOTIFY)) 
    {
		ret_val = bt_gatt_notify_cb(conn, &params);
	} 
    else 
    {
		ret_val = -EINVAL;
	}

    return ret_val;

}