#ifndef BTLE_H 
#define BTLE_H

#include <zephyr/types.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>


/** @brief NUS send statuses. */
enum bt_xlrt_send_status 
{
	BT_XLRT_SEND_STATUS_ENABLED,	
	BT_XLRT_SEND_STATUS_DISABLED,
};

/** @brief Structure of pointers to the callback functions for service events. :-( )
 * 
 * 	@brief received
 *  FIRST function is data received callback.
	 *
	 * The data has been received as a write request on the NUS RX
	 * Characteristic.
	 *
	 * @param[in] conn  Pointer to connection object that has received data.
	 * @param[in] data  Received data.
	 * @param[in] len   Length of received data.
     * 
     @brief sent
     SECOND function is Data sent callback.
	 *
	 * The data has been sent as a notification and written on the NUS TX
	 * Characteristic.
	 *
	 * @param[in] conn Pointer to connection object, or NULL if sent to all connected peers.
	 
    @brief send_enabled 
    THIRD fuinction = Send state callback. Indicates the CCCD descriptor status of the 
    XLRT TX characteristic.	 
	 * @param[in] status Send notification status.	
 * 
*/
struct bt_xlrt_cb 
{
	void (*received)(struct bt_conn *conn, const uint8_t *const data, uint16_t len);
	void (*sent)(struct bt_conn *conn);
	void (*send_enabled)(enum bt_xlrt_send_status status);

};

#define NORDIC_UUIDs

#ifdef NORDIC_UUIDs 
/** @brief UUID of the NUS Service. **/
#define XLRT_SERVICE_UUID_VAL \
	BT_UUID_128_ENCODE(0x6e400001, 0xb5a3, 0xf393, 0xe0a9, 0xe50e24dcca9e)

/** @brief UUID of the TX Characteristic. **/
#define XLRT_TX_UUID_VAL \
	BT_UUID_128_ENCODE(0x6e400003, 0xb5a3, 0xf393, 0xe0a9, 0xe50e24dcca9e)

/** @brief UUID of the RX Characteristic. **/
#define XLRT_RX_UUID_VAL \
	BT_UUID_128_ENCODE(0x6e400002, 0xb5a3, 0xf393, 0xe0a9, 0xe50e24dcca9e)

#else

#define XLRT_SERVICE_UUID_VAL \
	BT_UUID_128_ENCODE(0x2a1f963c, 0x143a, 0x4f7d, 0x9107, 0x6315196a02a6)
	

/** @brief UUID of the TX Characteristic. **/
#define XLRT_TX_UUID_VAL \
	BT_UUID_128_ENCODE(0x651135b0, 0xdfe9, 0x45a2, 0x95c1, 0x46552a8a77e2)


/** @brief UUID of the RX Characteristic. **/
#define XLRT_RX_UUID_VAL \
	BT_UUID_128_ENCODE(0xe7d6274e, 0xb765, 0x11e9, 0xa2a3, 0x2a2ae2dbcce4)

#endif

#define BT_UUID_XLRT_SERVICE   BT_UUID_DECLARE_128(XLRT_SERVICE_UUID_VAL)
#define BT_UUID_XLRT_RX       BT_UUID_DECLARE_128(XLRT_RX_UUID_VAL)
#define BT_UUID_XLRT_TX       BT_UUID_DECLARE_128(XLRT_TX_UUID_VAL)


#endif
